/* STM32 VL Discovery Library
 * LCD driver  LCD.c
 * 4 bit communication only
 * ver 2.0
 *  Created on: Nov 7, 2010
 *      Author: mard@ mcu.cz
 *  http: mcu.cz
 */

/* Includes ------------------------------------------------------------------*/
#include "LCD.h"

/** Private_TypesDefinitions	*/

/** Private_Defines			*/

/** Private_Macros			*/
#define WAIT __asm__("mov r8,r8\n\tmov r8,r8\n\tmov r8,r8\n\tmov r8,r8\n\t"\
                     "mov r8,r8\n\tmov r8,r8\n\tmov r8,r8\n\tmov r8,r8\n\t"\
                     "mov r8,r8\n\tmov r8,r8\n\tmov r8,r8\n\tmov r8,r8\n\t"\
                     "mov r8,r8\n\tmov r8,r8\n\tmov r8,r8\n\tmov r8,r8\n\t"\
                     "mov r8,r8\n\tmov r8,r8\n\tmov r8,r8\n\t")
/* fce mov r8,r8 se provede za 1 cyklus, pri 24Mhz za 41,66 ns
 * celkem je zde 19 instrukci + 5 na volani fce, tj. doba je 1us! */


/** Private_Variables			*/


/** Private_Functions			*/

/**
  * @brief  Waiting
  * @param  cas : time to wait in us
  * @retval None
  */
void delay_us(uint32_t cas)
{
	if (cas > 0 )
		for(; cas != 0; cas--)
		{
			WAIT;
		}
}



/**
  * @brief  Configure the GPIO Pins for LCD.
  * @param  None
  * @retval : None
  */
void GPIO_LCD_Inicializace(void)
{

  GPIO_InitTypeDef GPIO_InitStructure;

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);

  GPIO_InitStructure.GPIO_Pin = LCD_E | LCD_RS | LCD_D4 | LCD_D5
		  | LCD_D6 | LCD_D7 | GPIO_Pin_8 | GPIO_Pin_9;  // 8 a 9 jsou LED!!
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(LCD_PORT, &GPIO_InitStructure);
}

/**
  * @brief  LCD Init
  * @param  None
  * @retval : None
  */
void LCD_Init(void)
{
  // nastavime 8-bitovy komunikacni mod !! (protoze nevime zda uz displej byl prepnut nebo ne)

  GPIO_ResetBits(LCD_PORT, LCD_RS);						//RS=0:Instruction
  GPIO_ResetBits(LCD_PORT, LCD_E);						//E=0:Disable
  delay_us(200000);
  Posli4bitLCD(0b00110000);
  delay_us(8000);	// dle datasheetu cekat vice nez 4.1 ms
  GPIO_ResetBits(LCD_PORT, LCD_E);					//E=0:Disable
  Posli4bitLCD(0b00110000);
  delay_us(1000);
  GPIO_ResetBits(LCD_PORT, LCD_E);					//E=0:Disable
  Posli4bitLCD(0b00110000);
  delay_us(100);
  GPIO_ResetBits(LCD_PORT, LCD_E);					//E=0:Disable
  delay_us(100);

  // ted by mel byt display v 8-bitovem modu a definitivne ho prepneme do 4-bitoveho modu!
  Posli4bitLCD(0b00100000);
  delay_us(100);
  GPIO_ResetBits(LCD_PORT, LCD_E);					//E=0:Disable
  delay_us(100);
  // nyni jsme ve 4-bitovem rezimu
  // Prikaz 0 0 1 DL N F - -
  // N = pocet radek 1=2 radky, 0=1 radka
  // DL = bitu komunikace 1=8bitu, 0=4bity
  // F = Font 1=5x10bodu, 0=5x8 bodu
  PosliPrikazLCD(0b00101000);
  delay_us(100);
  // zvedej sam adresu, posunuj kurzor
  PosliPrikazLCD(0b00001110);
  delay_us(100);
  PosliPrikazLCD(0b00000110);			         // Move right, no shift
}

/**
  * @brief  Send 8-bit Commend to TCD
  * @param  command
  * @retval : None
  */
void PosliPrikazLCD(uint8_t cmd)	//v promenne cmd mame zaslany typ prikazu
{
	Posli4bitLCD(cmd);
	GPIO_ResetBits(LCD_PORT, LCD_E);					//E=0:Disable
	cmd = cmd<<4;
	Posli4bitLCD(cmd);
	GPIO_ResetBits(LCD_PORT, LCD_E);					//E=0:Disable
}


/**
  * @brief  Send 4-bit value to LCD
  * @param  command
  * @retval : None
  */
void Posli4bitLCD(uint8_t c)
{
	uint16_t Pom;
	c=c>>4;
	Pom = GPIO_ReadOutputData(LCD_PORT);
	Pom = Pom & 0xFFF0;
	Pom = Pom | (uint16_t)c;
	GPIO_Write(LCD_PORT, Pom);
	GPIO_SetBits(LCD_PORT, LCD_E);						//E=1:Enable
	delay_us(20);
}


/**
  * @brief  Send 4-bit Commend to TCD
  * @param  command
  * @retval : None
  */
void PosliZnakLCD(uint8_t cmd)	//v promnn cmd mme poslany znak na LCD
{
	GPIO_SetBits(LCD_PORT, LCD_RS);					//RS=1:Data
	Posli4bitLCD(cmd);
	GPIO_ResetBits(LCD_PORT, LCD_E);					//E=0:Disable
	delay_us(200);
	//GPIO_ResetBits(LCD_PORT, LCD_RS);				//RS=0:Instruction
	cmd = cmd<<4;
	//GPIO_SetBits(LCD_PORT, LCD_RS);					//RS=1:Data
	Posli4bitLCD(cmd);
	GPIO_ResetBits(LCD_PORT, LCD_E);					//E=0:Disable
	GPIO_ResetBits(LCD_PORT, LCD_RS);				//RS=0:Instruction
	delay_us(10);
}

/**
  * @brief  Send 4-bit Commend to TCD
  * @param  command
  * @retval : None
  */
void LCD_Clear(void)
{
    PosliPrikazLCD(0b00000001);
	delay_us(4000);
}

/**
  * @brief  Set LCD Cursor to Home
  * @param  None
  * @retval : None
  */
void LCD_CursorHome(void)
{
    PosliPrikazLCD(0b00000010);
	delay_us(1500);
}

/**
  * @brief  Set LCD Cursor to position X,Y
  * @param  sloupec, radka
  * @retval : None
  */
void set_cursor (int sloupec, int radka)
{
  PosliPrikazLCD(0x80 + (((radka * 40) + sloupec)& 0x7F));
}


/**
  * @brief  String variable print to LCD
  * @param  string pointer
  * @retval : None
  */
void printLCD(uint8_t *retezec)		//posila se odkaz na retezec
{
	uint8_t i=0;
	// pro sichr si zkontrolujeme pointer
	if (!retezec) return;
	while(retezec[i]!='\0')
	{
		PosliZnakLCD(retezec[i]);
		i++;
	}
}
/**
  * @brief  String constant print to LCD
  * @param  string pointer
  * @retval : None
  */
void printLCDc(const uint8_t *retezec)	//posila se odkaz na retezec z flash
{
	uint8_t i=0;
	// pro sichr si zkontrolujeme pointer
	if (!retezec) return;
	while(retezec[i]!='\0')
	{
		PosliZnakLCD(retezec[i]);
		i++;
	}
	delay_us(300);
}
