/* Includes ------------------------------------------------------------------*/
#include <stddef.h>
#include <string.h>
#include "stm32f10x.h"
#include "STM32_Discovery.h"
#include "LCD.h"
/* Private typedef -----------------------------------------------------------*/
GPIO_InitTypeDef GPIO_InitStructure;

/* Private variables ---------------------------------------------------------*/
uint8_t LCD_text[] = "PROJEKT MMIA";
uint16_t Hodnota=0;
__IO uint8_t c;
__IO uint16_t Perioda = 100;	// budeme mit celkem 100 dilku na periodu
__IO uint16_t Counter = 0;		// interni citac generatoru probehu
__IO uint16_t Hodnota_DAC1 = 0;


/* Private function prototypes -----------------------------------------------*/
void GPIO_Inicializace(void); // nastaven vstupne/vystupnich pinu na kitu
void ADC_Inicializace(void);
void Delay(__IO uint32_t nTick);
uint16_t ADC1_Read(void);
void itoa(uint16_t n, int8_t s[]);
void reverse(int8_t s[]);
void Delay(__IO uint32_t Doba); // Doba = v milisekundach
void DAC_Inicializace(void);
void Generuj_DAC(void);
/* Private functions ---------------------------------------------------------*/

int main(void)
{
  GPIO_LCD_Inicializace();
  STM32_Discovery_LEDOn(LED4);	 // LED4 - modra
  Delay(0xAAAA);
  GPIO_Inicializace();
  DAC_Inicializace();
  LCD_Init();
  LCD_Clear();
  ADC_Inicializace();
  printLCD(LCD_text);
  Delay(0xFFFFF);
  STM32_Discovery_LEDOff(LED4); // LED4 - modra

  while (1)
  {
	  Generuj_DAC();
	  Delay(0xAFFFF);
	  LCD_Clear();
	  printLCD("U=\0");
	  Hodnota = (ADC1_Read() * 732)/10000;
	  itoa(Hodnota/100,LCD_text);				//PEVEDE SLO NA TEXT
	  printLCD(LCD_text);
	  printLCD(".\0");
	  itoa(Hodnota % 100,LCD_text);
	  printLCD(LCD_text);
	  printLCD(" V\0");
  }
}

void GPIO_Inicializace(void)			//inicializace port pro ledky a tlatka
{
	  STM32_Discovery_LEDInit(LED3);
	  STM32_Discovery_LEDInit(LED4);
	  STM32_Discovery_PBInit(BUTTON_USER, BUTTON_MODE_GPIO);
	  STM32_Discovery_LEDOff(LED3);
	  STM32_Discovery_LEDOff(LED4);
}


void DAC_Inicializace(void)
{

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);				//piveden hodin na modul brny A (=aktivace brny)
  /* DAC Periph clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);				//piveden hodni na modul D/A pevodnku (=aktivace pevodnku)

  GPIO_InitTypeDef GPIO_InitStructure;								//vytvoen struktury pro prci s brnou
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4; 						//pin 4 je zvolen pin 4 jako vstup
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;						//pipojen pinu 4 k D/A pevodnku
  GPIO_Init(GPIOA, &GPIO_InitStructure);							//penesen nastavench parametr na portA

  DAC_InitTypeDef            DAC_InitStructure;						//vytvoen sruktury DA pevodnku

  DAC_InitStructure.DAC_Trigger = DAC_Trigger_None;					//deaktivace trggeru (manuln spoutn)
  DAC_InitStructure.DAC_WaveGeneration = DAC_WaveGeneration_None;	//deakitivace internho genrtoru funkc
  DAC_InitStructure.DAC_OutputBuffer = DAC_OutputBuffer_Enable;		//pipojen vstupnho buffru pro  zajitn pevnho napt

  DAC_Init(DAC_Channel_1, &DAC_InitStructure);						//zaveden nastaven struktury pro 1 kanl DA pevodnku
    DAC_Cmd(DAC_Channel_1, ENABLE);									//aktivace kanlu

}

void Generuj_DAC(void)
{
	STM32_Discovery_LEDOff(LED4); // Vypneme LED4 - modrou
	if (Counter <= (Perioda /2))
	  {
		Hodnota_DAC1 = ((uint32_t)Counter)*2 * 0xFFF / Perioda;
	  }
	else
	  {
		Hodnota_DAC1 = 0xFFF - (((uint32_t)Counter - (Perioda /2))*2 * 0xFFF / Perioda);
	  }

	/* Set DAC Channel1 DAC_DHR12R1 register */
	DAC_SetChannel1Data(DAC_Align_12b_R, (uint32_t)Hodnota_DAC1);


	Counter++;
	if (Counter >= Perioda)			//nulovn tae periody + rozsvcen ledky
	  {
		Counter = 0;
		STM32_Discovery_LEDOn(LED4); // zapneme LED4 - modra - signal zacatku periody
	  }
}




void ADC_Inicializace(void)
{
  ADC_InitTypeDef  ADC_InitStructure;							//inicializace struktury A/D pevodnku
  GPIO_InitTypeDef GPIO_InitStructure;							//inicializace struktury pro i/o brny

  //* ADCCLK = PCLK2/1 = 24/2 = 12MHz*/
  RCC_ADCCLKConfig(RCC_PCLK2_Div2);								//nastaven frekvence A/D odpovd 12MHz

//nastaven pslunho portu
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);			//piveden hodin na modul brny C (=aktivace brny)
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;						//nastaven struktury A/D pevodnku aktivace pin4
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;				//nastaven kmitotu vnitnch hodin
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;					//nastaven reimu na analogov
  GPIO_Init(GPIOC, &GPIO_InitStructure);						//penesen nastaven konstrukce na brnu C

  /* povolime hodiny do A/D prevodniku */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);			//piveden hodin na modul A/D pevodnku (=aktivace modulu)

//ADC_DeInit(ADC1);

  /* ADC1 Configuration ------------------------------------------------------*/
  ADC_InitStructure.ADC_Mode                = ADC_Mode_Independent;				//reim nezvislho pevodu
  ADC_InitStructure.ADC_ScanConvMode        = DISABLE;							//dekativace ?
  ADC_InitStructure.ADC_ContinuousConvMode  = DISABLE;							//deaktivace ?
  ADC_InitStructure.ADC_ExternalTrigConv    = ADC_ExternalTrigConv_None;		//vypnout extern spooutn
  ADC_InitStructure.ADC_DataAlign           = ADC_DataAlign_Right;				//zarovnn dat nevy v pravo ??
  ADC_InitStructure.ADC_NbrOfChannel        = 1;								//volba kanlu
  ADC_Init( ADC1, &ADC_InitStructure );											//penesen struktruy do A/D pevodnku

  /* ADC1 Regular Channel 14 */
  ADC_RegularChannelConfig( ADC1, ADC_Channel_14,  1, ADC_SampleTime_239Cycles5);


  /* Enable ADC1 external trigger conversion */
  ADC_ExternalTrigConvCmd( ADC1, ENABLE );

  ADC_Cmd(ADC1, ENABLE);

  /* SEKVENCE KALIBROVN a/d PEVODNKU */
  /* Enable ADC1 reset calibration register */
  ADC_ResetCalibration(ADC1);
  /* Check the end of ADC1 reset calibration register */
  while(ADC_GetResetCalibrationStatus(ADC1));
  /* Start ADC1 calibration */
  ADC_StartCalibration(ADC1);
  /* Check the end of ADC1 calibration */
  while(ADC_GetCalibrationStatus(ADC1));
}

uint16_t ADC1_Read(void)
{
  // Start the conversion
  ADC_SoftwareStartConvCmd(ADC1, ENABLE);
  // Wait until conversion completion
  while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
  // Get the conversion value
  return ADC_GetConversionValue(ADC1);
}


void Delay(__IO uint32_t nTick)
{
  for(; nTick != 0; nTick--);
}


void itoa(uint16_t n, int8_t s[])
{
    int i, sign;
    if ((sign = n) < 0)  /* record sign */
        n = -n;          /* make n positive */
    i = 0;
    do {       /* generate digits in reverse order */
        s[i++] = n % 10 + '0';   /* get next digit */
    } while ((n /= 10) > 0);     /* delete it */
    if (sign < 0)
        s[i++] = '-';
    s[i] = '\0';
    reverse(s);
}


void reverse(int8_t s[])
{
    int i, j;
    char c;

    for (i = 0, j = strlen(s)-1; i<j; i++, j--) {
        c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}


/*
 * Minimal __assert_func used by the assert() macro
 * */
void __assert_func(const char *file, int line, const char *func, const char *failedexpr)
{
  while(1)
  {}
}

/*
 * Minimal __assert() uses __assert__func()
 * */
void __assert(const char *file, int line, const char *failedexpr)
{
   __assert_func (file, line, NULL, failedexpr);
}
